@@ -1551,10 +1551,17 @@ function makeRetainedCompilerSettings() {
 var WASM_PAGE_SIZE = 65536;
 var ASMJS_PAGE_SIZE = 16777216;
 
-function getPageSize() {
+function getMemoryPageSize() {
   return WASM ? WASM_PAGE_SIZE : ASMJS_PAGE_SIZE;
 }
 
+// Page size reported by some POSIX calls, mostly filesystem. This does not
+// depend on the memory page size which differs between wasm and asm.js, and
+// makes us report a consistent value despite the compile target. However,
+// perhaps we should unify all the page sizes (especially after fastcomp is
+// gone TODO).
+var POSIX_PAGE_SIZE = 16384;
+
 // Receives a function as text, and a function that constructs a modified
 // function, to which we pass the parsed-out name, arguments, and body of the
 // function. Returns the output of that function.
