@@ -5,8 +5,7 @@
  * found in the LICENSE file.
  */
 
-#ifndef __emscripten_events_h__
-#define __emscripten_events_h__
+#pragma once
 
 #ifdef __cplusplus
 #if !defined(__DEFINED_pthread_t)
@@ -131,7 +130,6 @@ extern EMSCRIPTEN_RESULT emscripten_set_keydown_callback_on_thread(const char *t
 extern EMSCRIPTEN_RESULT emscripten_set_keyup_callback_on_thread(const char *target, void *userData, EM_BOOL useCapture, em_key_callback_func callback, pthread_t targetThread);
 
 typedef struct EmscriptenMouseEvent {
-  double timestamp;
   long screenX;
   long screenY;
   long clientX;
@@ -211,7 +209,6 @@ extern EMSCRIPTEN_RESULT emscripten_set_focusin_callback_on_thread(const char *t
 extern EMSCRIPTEN_RESULT emscripten_set_focusout_callback_on_thread(const char *target, void *userData, EM_BOOL useCapture, em_focus_callback_func callback, pthread_t targetThread);
 
 typedef struct EmscriptenDeviceOrientationEvent {
-  double timestamp;
   double alpha;
   double beta;
   double gamma;
@@ -224,8 +221,11 @@ extern EMSCRIPTEN_RESULT emscripten_set_deviceorientation_callback_on_thread(voi
 
 extern EMSCRIPTEN_RESULT emscripten_get_deviceorientation_status(EmscriptenDeviceOrientationEvent *orientationState);
 
+#define EMSCRIPTEN_DEVICE_MOTION_EVENT_SUPPORTS_ACCELERATION                   0x01
+#define EMSCRIPTEN_DEVICE_MOTION_EVENT_SUPPORTS_ACCELERATION_INCLUDING_GRAVITY 0x02
+#define EMSCRIPTEN_DEVICE_MOTION_EVENT_SUPPORTS_ROTATION_RATE                  0x04
+
 typedef struct EmscriptenDeviceMotionEvent {
-  double timestamp;
   double accelerationX;
   double accelerationY;
   double accelerationZ;
@@ -235,6 +235,7 @@ typedef struct EmscriptenDeviceMotionEvent {
   double rotationRateAlpha;
   double rotationRateBeta;
   double rotationRateGamma;
+  int supportedFields;
 } EmscriptenDeviceMotionEvent;
 
 
@@ -486,6 +487,22 @@ extern EMSCRIPTEN_RESULT emscripten_webgl_commit_frame(void);
 
 extern EM_BOOL emscripten_supports_offscreencanvas(void);
 
+// Returns function pointers to WebGL 1 functions. Please avoid using this function ever - all WebGL1/GLES2 functions, even those for WebGL1 extensions, are available to user code via static linking. Calling GL functions
+// via function pointers obtained here is slow, and using this function can greatly increase resulting compiled program size. This functionality is available only for easier program code porting purposes, but be aware
+// that calling this is causing a noticeable performance and compiled code size hit.
+extern void *emscripten_webgl1_get_proc_address(const char *name);
+
+// Returns function pointers to WebGL 2 functions. Please avoid using this function ever - all WebGL2/GLES3 functions, even those for WebGL2 extensions, are available to user code via static linking. Calling GL functions
+// via function pointers obtained here is slow, and using this function can greatly increase resulting compiled program size. This functionality is available only for easier program code porting purposes, but be aware
+// that calling this is causing a noticeable performance and compiled code size hit.
+extern void *emscripten_webgl2_get_proc_address(const char *name);
+
+// Combines emscripten_webgl1_get_proc_address() and emscripten_webgl2_get_proc_address() to return function pointers to both WebGL1 and WebGL2 functions. Same drawbacks apply.
+extern void *emscripten_webgl_get_proc_address(const char *name);
+
+typedef struct WGPUDeviceImpl* WGPUDevice;
+extern WGPUDevice emscripten_webgpu_get_device();
+
 extern EMSCRIPTEN_RESULT emscripten_set_canvas_element_size(const char *target, int width, int height);
 extern EMSCRIPTEN_RESULT emscripten_get_canvas_element_size(const char *target, int *width, int *height);
 
@@ -560,8 +577,8 @@ extern void emscripten_console_error(const char *utf8String);
 extern void emscripten_throw_number(double number);
 extern void emscripten_throw_string(const char *utf8String);
 
+extern void emscripten_unwind_to_js_event_loop(void) __attribute__((noreturn));
+
 #ifdef __cplusplus
 } // ~extern "C"
 #endif
-
-#endif
